% Richard Crump and Nikolay Gospodinov
% "On the Factor Structure of Bond Returns"
% May 31, 2021

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Replication Code: Figure 1
% Note: See README for further details
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
clc;
close all;
% Plots directory
%!% Change path
filePath = '../Drafts/Plots';

tmpNom = readtable('feds200628.csv', 'Range', 'A10');

idx = ~isnan(tmpNom.BETA0(:,1));
dates = datenum(tmpNom.Date);
dates = dates(idx);
Beta = [tmpNom.BETA0, tmpNom.BETA1, tmpNom.BETA2, tmpNom.BETA3];
Beta = Beta(idx,:);
Tau = [tmpNom.TAU1, tmpNom.TAU2];
Tau = Tau(idx,:);
mdy = tmpNom.Date;
mdy = mdy(idx);
Beta(Beta==-999.99) = 0;
Tau(Tau==-999.99) = 0;

idx1 = find(datenum(mdy) == datenum(1992,4,1));
idx2 = find(datenum(mdy) == datenum(2020,6,30));

Beta = Beta(idx1:idx2,:);
Tau = Tau(idx1:idx2,:);
mdy = mdy(idx1:idx2);

n = (1:1:360)/12;
novertau1 = (1./Tau(:,1))*n;
novertau2 = (1./Tau(:,2))*n;

yNom1 = Beta(:,1);
yNom2 = Beta(:,2).*(1-exp(-novertau1))./novertau1;
yNom3 = Beta(:,3).*((1-exp(-novertau1))./(novertau1)-exp(-novertau1));
yNom4 = Beta(:,4).*((1-exp(-novertau2))./(novertau2)-exp(-novertau2));
yNom = yNom1+yNom2+yNom3+yNom4;

pNom = -(ones(numel(mdy),1)*n*3).*(4*yNom/1200);

i = 1;
idxQ = [];

while (mdy(i)<mdy(end))
    if any(intersect(month(mdy(i)), [3,6,9,12]))
        if (day(mdy(i))-15<-4) || (day(mdy(i))-15 > 0)
            i = i + 1;
        else
            % Add 10* so never choose day larger than 15
            tmp = abs(day(mdy(i:i+4))-15) + (10*(day(mdy(i:i+4))>15));
            [~,tmpidx] = min(tmp);
            idxQ = [idxQ; i+tmpidx-1];
            i = i+5;
        end        
    else
       i = i + 1; 
    end
   
end

pNomQLastAll = pNom(idxQ,:);
mdyQLast = mdy(idxQ,:);

pNomQLast = pNomQLastAll(:,3:3:60);
fNomQLast = pNomQLast(:,1:end-1)-pNomQLast(:,2:end);
fNomQLast = 400*[-pNomQLast(:,1), fNomQLast];

retNomQLast = 400*(pNomQLast(2:end,1:end-1) - pNomQLast(1:end-1,2:end));
xretNomQLast = 400*(pNomQLast(2:end,1:end-1) - pNomQLast(1:end-1,2:end) + pNomQLast(1:end-1,1));
dretNomQLast = [xretNomQLast(:,1), (xretNomQLast(:,2:end)-xretNomQLast(:,1:end-1))];

[DretLoadings, DretEigenvalues] = OrderedPCA(dretNomQLast,'cov');
[XretLoadings, XretEigenvalues] = OrderedPCA(xretNomQLast,'cov');

figName = figure(1);
plot(datenum(mdyQLast(2:end)), xretNomQLast, 'Color', .3*[1,1,1]); ylim([-30,30]); 
set(gca,'FontSize',16);
datetick('x', 'yyyy','keeplimits');
xlim([datenum(1992,3,1), datenum(2020,12,31)]);
title('Excess Returns');
try print(figName, '-dpdf', [outputPath, 'xret.pdf']); end

figName = figure(2);
plot(datenum(mdyQLast(2:end)), dretNomQLast, 'Color', .3*[1,1,1]); ylim([-3,3]); 
set(gca,'FontSize',16);
datetick('x', 'yyyy','keeplimits');
xlim([datenum(1992,3,1), datenum(2020,12,31)]);
title('Difference Returns');
try print(figName, '-dpdf', [outputPath, 'dret.pdf']); end

figName = figure(3);
plot(2:20, XretLoadings(:,1), 'k', 'LineWidth', 1.5);
hold on
plot(2:20, XretLoadings(:,2)', '--k', 'LineWidth', 1.5);
plot(2:20, XretLoadings(:,3)', ':k', 'LineWidth', 1.5);
hold off
set(gca,'FontSize',16);
legend({'PC1', 'PC2', 'PC3'}, 'Interpreter', 'latex', 'Location', 'SouthWest');
xlim([2,20]);
text(4.5, .455,['Variation Explained by first 3 PCs: ', num2str(round(100*sum(XretEigenvalues(1:3))./sum(XretEigenvalues),2)), '\%'], ...
        'FontSize',16, 'Interpreter', 'latex');
ylim([-0.4,0.5]);
xlabel('Maturity in Quarters', 'Interpreter', 'latex');
title('Excess Returns: PC Loadings');
try print(figName, '-dpdf', [outputPath, 'xretLoadings.pdf']); end

figName = figure(4);
plot(2:20, DretLoadings(:,1), 'k', 'LineWidth', 1.5);
hold on
plot(2:20, DretLoadings(:,2), '--k', 'LineWidth', 1.5);
plot(2:20, DretLoadings(:,3),  ':k', 'LineWidth', 1.5);
hold off
set(gca,'FontSize',16);
legend({'PC1', 'PC2', 'PC3'}, 'Interpreter', 'latex', 'Location', 'SouthWest');
xlim([2,20]);
text(4.5, .555,['Variation Explained by first 3 PCs: ', num2str(round(100*sum(DretEigenvalues(1:3))./sum(DretEigenvalues),2)), '\%'], ...
        'FontSize',16, 'Interpreter', 'latex');
ylim([-0.3,0.6]);
xlabel('Maturity in Quarters', 'Interpreter', 'latex');
title('Difference Returns: PC Loadings');
try print(figName, '-dpdf', [outputPath, 'dretLoadings.pdf']); end

figName = figure(5);
plot(1:10, XretEigenvalues(1:10)./sum(XretEigenvalues), '-kd',  'LineWidth', 1.5);
set(gca,'FontSize',16);
ylim([0,1]);
xlim([1,10]);
xlabel('Principal Component', 'Interpreter', 'latex');
title('Excess Returns: Scree Plot');
try print(figName, '-dpdf', [outputPath, 'xretExplained.pdf']); end

figName = figure(6);
plot(1:10, DretEigenvalues(1:10)./sum(DretEigenvalues), '-kd',  'LineWidth', 1.5);
set(gca,'FontSize',16);
ylim([0,1]);
xlim([1,10]);
xlabel('Principal Component', 'Interpreter', 'latex');
title('Difference Returns: Scree Plot');
try print(figName, '-dpdf', [outputPath, 'dretExplained.pdf']); end



