% Richard Crump and Nikolay Gospodinov
% "On the Factor Structure of Bond Returns"
% May 31, 2021

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Replication Code: Figure 2
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
clc;
% Plots directory
%!% Change path
filePath = '../Drafts/Plots';

% Parameters
sig2 = 2;
N = 20;
% Generate matrix of interest
L = tril(ones(N));
V_D = sig2*(L*L');

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Numerical eigenvalues/eigenvectors 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[numEigvec, numEigval] = OrderedPCA(V_D);

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Theoretical eigenvalues/eigenvectors 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
thEigval = NaN(N,1);
for n = 1:N
    thEigval(n) = sig2/(2-2*cos((2*n-1)*pi/(2*N+1)));
end
thEigvec = NaN(N,N);
for c = 1:N % columns
    for r = 1:N % rows
        thEigvec(r,c) = 2*sin(r*(2*c-1)*pi/(2*N+1))/sqrt(2*N+1);       
    end
end
% Ensure first element of eigenvector positive
thEigvec = repmat(sign(thEigvec(1,:)),[N,1]).*thEigvec;

% Confirm numerical and theoretical equivalence
%{
sum(sum(abs(V_D-numEigvec*diag(numEigval)*numEigvec')))
sum(sum(abs(V_D-thEigvec*diag(thEigval)*thEigvec')))
sum(sum(abs(numEigvec-thEigvec)))
sum(abs(numEigval-thEigval))
%}

figName = figure;
plot(thEigvec(:,1), 'k', 'LineWidth', 1.5)
hold on
plot(thEigvec(:,2), '--k', 'LineWidth', 1.5)
plot(thEigvec(:,3), ':k', 'LineWidth', 1.5)
hold off
set(gca,'FontSize',16);
legend({'PC1', 'PC2', 'PC3'}, 'Interpreter', 'latex', 'location', 'southwest');
text(2.5, .365,['Variation Explained by first 3 PCs: ', num2str(round(100*sum(thEigval(1:3)/sum(thEigval)),2)), '\%'], ...
        'FontSize',16, 'Interpreter', 'latex');
xlabel('Maturity', 'Interpreter', 'latex');
try print(figName, '-dpdf', [filePath, filesep, 'ret0LoadingsTheoretical.pdf']); end

figName = figure;
line([1,10], [1/N,1/N], 'Color', .5*[1,1,1], 'LineStyle', '-', 'LineWidth', 1.5);
hold on
plot(thEigval(1:10)/sum(thEigval), '-kd','LineWidth', 1.5);
set(gca,'FontSize',16);
ylim([0,1]);
xlim([1,10]);
hold off
xlabel('Principal Component', 'Interpreter', 'latex');
box on;
try print(figName, '-dpdf', [filePath, filesep, 'ret0ScreeTheoretical.pdf']); end

