% Richard Crump and Nikolay Gospodinov
% "On the Factor Structure of Bond Returns"
% May 31, 2021

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Replication Code: Figure 3
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
clc;
% Plots directory
%!% Change path
filePath = '../Drafts/Plots';

% Parameters
sig2 = 2;
N = 20;
rho = .99;
% Generate matrix of interest
T0 = toeplitz(rho.^(0:1:N-1));
H0 = hankel(rho.^(0:1:N-1),rho.^(N-1:1:2*N-2));
% Covariance matrix of interest
V_D = sig2*(T0-(rho^2)*H0)/(1-rho^2);
% Approximating matrix
Tri0 = full(gallery('tridiag',N,-rho/sig2,(1+rho^2)/sig2,-rho/sig2));
Tri0(end,end) = (1-rho+rho^2)/sig2;
V_Dtilde = inv(Tri0);

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Numerical eigenvalues/eigenvectors 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[numEigvec, numEigval] = OrderedPCA(V_D);

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Theoretical eigenvalues/eigenvectors 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
thEigval = NaN(N,1);
for n = 1:N
    thEigval(n) = sig2/(1+rho^2-2*rho*cos((2*n-1)*pi/(2*N+1)));
end
thEigvec = NaN(N,N);
for c = 1:N % columns
    for r = 1:N % rows
        thEigvec(r,c) = 2*sin(r*(2*c-1)*pi/(2*N+1))/sqrt(2*N+1);
    end
end
% Ensure first element of eigenvector positive
thEigvec = repmat(sign(thEigvec(1,:)),[N,1]).*thEigvec;

%{
Check:
sum(sum(abs(V_D-numEigvec*diag(numEigval)*numEigvec')))
sum(sum(abs(V_Dtilde-thEigvec*diag(thEigval)*thEigvec')))
%}

thTraceV_Dtilde = (1/2)*(2*N+1)*(1-rho^(2*N+1))/((1+rho^(2*N+1))*(1-rho^2)) - (1/2)/(1+2*rho+rho^2);
thTraceV_Dtilde = sig2*thTraceV_Dtilde;

%{
Check:
abs(trace(V_Dtilde) - thTraceV_Dtilde)
%}

figName = figure(1);
p1 = plot(thEigvec(:,1), 'color', .5*[1,1,1], 'LineStyle', '-', 'LineWidth', 1.25);
hold on
p2 = plot(numEigvec(:,1), 'color', 'k', 'LineWidth', 1.5);
p3 = plot(thEigvec(:,2), 'color', .5*[1,1,1], 'LineStyle', '--', 'LineWidth', 1.25);
p4 = plot(numEigvec(:,2), '--k', 'LineWidth', 1.5);
p5 = plot(thEigvec(:,3), 'color', .5*[1,1,1], 'LineStyle', ':', 'LineWidth', 1.25);
p6 = plot(numEigvec(:,3), ':k', 'LineWidth', 1.5);
hold off
set(gca,'FontSize',16);
legend([p2,p1,p4,p3,p6,p5],{'PC1', 'PC1 Approx.', 'PC2', 'PC2 Approx.', 'PC3', 'PC3 Approx.'}, 'Interpreter', 'latex', 'location', 'southwest');
text(2.5, .365,['Variation Explained by first 3 PCs: ', num2str(round(100*sum(numEigval(1:3)/sum(numEigval)),2)), '\%'], ...
        'FontSize',16, 'Interpreter', 'latex');
xlabel('Maturity', 'Interpreter', 'latex');
try print(figName, '-dpdf', [filePath, filesep, 'dretLoadingsTheoretical.pdf']); end

figName = figure(2);
plot(numEigval(1:10)/sum(numEigval), '-kd','LineWidth', 1.5);
hold on
plot(thEigval(1:10)/sum(thEigval), 'k:', 'LineWidth', 1.5);
set(gca,'FontSize',16);
ylim([0,1]);
xlim([1,10]);
legend('Share Explained', 'Share Explained Approx.', 'Interpreter', 'latex')
xlabel('Principal Component', 'Interpreter', 'latex');
try print(figName, '-dpdf', [filePath, filesep, 'dretScreeTheoretical.pdf']); end

   