% Richard Crump and Nikolay Gospodinov
% "On the Factor Structure of Bond Returns"
% May 31, 2021

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Replication Code: Figure 4
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
clc;
% Plots directory
%!% Change path
filePath = '../Drafts/Plots';

% Parameters
sig2 = 2;
N = 20;
rho = .99;
% Generate matrix of interest
T0 = toeplitz(rho.^(0:1:N-1));
H0 = hankel(rho.^(0:1:N-1),rho.^(N-1:1:2*N-2));
% Covariance matrix of interest
V_D = sig2*(T0-(rho^2)*H0)/(1-rho^2);
% Approximating matrix
Tri0 = full(gallery('tridiag',N,-rho/sig2,(1+rho^2)/sig2,-rho/sig2));
Tri0(end,end) = (1-rho+rho^2)/sig2;
V_Dtilde = inv(Tri0);
%
L = tril(ones(N));
V_R = L*V_D*L';
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Numerical eigenvalues/eigenvectors 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[numEigvec, numEigval] = OrderedPCA(V_R);

figName = figure;
plot(numEigvec(:,1), 'k', 'LineWidth', 1.5)
hold on
plot(numEigvec(:,2), '--k', 'LineWidth', 1.5)
plot(numEigvec(:,3), ':k', 'LineWidth', 1.5)
hold off
set(gca,'FontSize',16);
legend({'PC1', 'PC2', 'PC3'}, 'Interpreter', 'latex', 'location', 'southwest');
text(2.5, .435,['Variation Explained by first 3 PCs: ', num2str(round(100*sum(numEigval(1:3)/sum(numEigval)),2)), '\%'], ...
        'FontSize',16, 'Interpreter', 'latex');
xlabel('Maturity', 'Interpreter', 'latex');
try print(figName, '-dpdf', [filePath, filesep, 'retLoadingsTheoretical.pdf']); end


figName = figure;
plot(numEigval(1:10)/sum(numEigval), '-kd','LineWidth', 1.5);
set(gca,'FontSize',16);
ylim([0,1]);
xlim([1,10]);
xlabel('Principal Component', 'Interpreter', 'latex');
try print(figName, '-dpdf', [filePath, filesep, 'retScreeTheoretical.pdf']); end

