% Richard Crump and Nikolay Gospodinov
% "On the Factor Structure of Bond Returns"
% May 31, 2021

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Replication Code: Figure 5 (Example 1)
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
clc;
% Plots directory
%!% Change path
filePath = '../Drafts/Plots';
N = 20;
sig2 = 2;
rho = [.5, .75, .99];

% mu is obtained from Eurodollars data as described in the paper
mu = [0.1012    0.2490    0.4211    0.6125    0.8080    1.0032    1.1905    1.3712    1.5441    1.7126 ...
    1.8759    2.0369    2.1926    2.3448    2.4940    2.4534    2.5874    2.7183    2.8463    2.8920]';

ratio = NaN(N,numel(rho));

for r = 1:numel(rho)
    % Generate matrix of interest
    T0 = toeplitz(rho(r).^(0:1:N-1));
    H0 = hankel(rho(r).^(0:1:N-1),rho(r).^(N-1:1:2*N-2));
    % Covariance matrix of interest
    V_D = sig2*(T0-(rho(r)^2)*H0)/(1-rho(r)^2);
    C1 = tril(ones(N));
    V_R = C1*V_D*C1';

    [eigvecVRnum,eigvalVRnum] = OrderedPCA(V_R);

    VRapprox = NaN([size(V_R),N]);
    for m = 1:N
        tmp = 0;
        for n = 1:N
            if n <= m
                tmp = tmp + eigvalVRnum(n)*eigvecVRnum(:,n)*eigvecVRnum(:,n)';            
            else
                tmp = tmp + eye(N)*trace(eigvalVRnum(n)*eigvecVRnum(:,n)*eigvecVRnum(:,n)')/(N-m);
            end
        end
        VRapprox(:,:,m) = tmp;
    end

    for i = 3:N
       sharpesubopt = mu'*inv(VRapprox(:,:,i))*mu/(sqrt(mu'*inv(VRapprox(:,:,i))*V_R*inv(VRapprox(:,:,i))*mu));
       sharpeopt = sqrt(mu'*inv(V_R)*mu);
       ratio(i,r) = sharpesubopt/sharpeopt;
    end
end

figName = figure;
set(0,'DefaulttextInterpreter','latex');
plot(ratio(:,1), 'k', 'linewidth', 1.25)
hold on
plot(ratio(:,2), '--k' ,'linewidth', 1.25)
plot(ratio(:,3), ':k', 'linewidth', 1.25)
hold off
ylim([.3 1])
legend({'$\rho=0.50$','$\rho=0.75$','$\rho=0.99$'},'Interpreter','latex','location','SE')
xlabel('Number of Principal Components')
ylabel('Ratio')
set(gca,'FontSize',16);
try print(figName, '-dpdf', [filePath, filesep, 'graphExample1.pdf']); end
