% Richard Crump and Nikolay Gospodinov
% "On the Factor Structure of Bond Returns"
% May 31, 2021

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Replication Code: Figure 5 (Example 2)
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
clc;
% Plots directory
%!% Change path 
filePath = '../Drafts/Plots';

N = 20;
sig2 = 2;
rho = [0.5, .75, .99];
m = 3;

C1 = tril(ones(N));

corr = NaN(N,numel(rho));
for r = 1:numel(rho)
    % Generate matrix of interest
    T0 = toeplitz(rho(r).^(0:1:N-1));
    H0 = hankel(rho(r).^(0:1:N-1),rho(r).^(N-1:1:2*N-2));
    % Covariance matrix of interest
    V_D = sig2*(T0-(rho(r)^2)*H0)/(1-rho(r)^2);    
    V_R = C1*V_D*C1';

    [eigvecVRnum,eigvalVRnum] = OrderedPCA(V_R);

    for i = 1:N
        w = zeros(N,1);
        w(i) = 1;

        wf = inv(eigvecVRnum(:,1:m)'*V_R*eigvecVRnum(:,1:m))*eigvecVRnum(:,1:m)'*V_R*w;        

        tmpcorr = w'*V_R*eigvecVRnum(:,1:m)*wf/sqrt(w'*V_R*w*wf'*eigvecVRnum(:,1:m)'*V_R*eigvecVRnum(:,1:m)*wf);

        corr(i,r) = tmpcorr;

    end
end

figName = figure;
set(0,'DefaulttextInterpreter','latex');
plot(corr(:,1), 'k', 'linewidth', 1.25);
hold on
plot(corr(:,2), '--k', 'linewidth', 1.25);
plot(corr(:,3), ':k', 'linewidth', 1.25);
hold off
ylim([0.5,1])
legend({'$\rho=0.50$','$\rho=0.75$','$\rho=0.99$'},'Interpreter','latex','location','SE')
xlabel('Target Portfolio')
ylabel('Correlation with Hedging Portfolio')
set(gca,'FontSize',16);
try print(figName, '-dpdf', [filePath, filesep, 'graphExample2.pdf']); end
